function [URL, URLinfo] = ssm35url(archive, URLinfo, LonRange, Nlon, ...
    LatRange, Nlat, dset_stride, ranges, variable, server)
%----------------------------------------------------------------------------
%
% Formal parameters:
% archive: Name of .m file containing metadata about this dataset. 
% LonRange, LatRange, Nlon, Nlat -- used to construct URL constraints
% for gridded data.
% dset_stride: The stride value to use when reading the data. 
%
% NB: pass_times: This is a matrix of time with the year, day and
% ascending/descending flag of each hit. Note that the day here is the
% calendar day and the flag uses 0 for ascending and 1 for descending.

% The return values are:
% URL: A vector of URLs, one for each element of pass_times.
%
%----------------------------------------------------------------------------

% Note:  Inflexibly coding the url like this isn't quite proper, but in
%        the absence of a "real" catalog server - we've going to put up
%        with it for now.  The url should come from the catalog server
%        and it is, in fact, stored there - but that string is ignored.

server = deblank(server);

% extract needed info from catalog database.
URL = '';
if ~isfield('URLinfo','info')
  return
end
info = URLinfo.info;

if ~isempty(info)
  % we need year, last two digits of year and first three digits
  % of the month to build the rest of the path to the data.
  % What is the year ?
  year = num2str(info(:,1));
  % what is the month ?
  month = info(:,2);
  monthList = str2mat('jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', ...
      'aug', str2mat('sep', 'oct', 'nov', 'dec'));
  monstr = monthList(month,:);
  y2kyear = year(:,3:4);
  
  numurls = size(month,1);
  URL = cell(numurls,1);
  %figure out if wraparound is needed.
  [start_lon,end_lon] = splitrequest(LonRange, ranges(1,:));
  rows = pixcheck(ranges(2,:), LatRange, Nlat);
  rows = rows - 1;
  
  urlcon = [];
  for k = 1:numurls
    for j = 1:length(start_lon);
      columns{j} = pixcheck([start_lon(j) end_lon(j)], LonRange, ...
	  Nlon);
      columns{j} = columns{j} - 1;
      % Change the row/col  constraint information to string.
      StartRow = num2str(rows(1));
      stride = num2str(dset_stride);
      EndRow = num2str(rows(2));
      StartColumn = num2str(columns{j}(1));
      EndColumn = num2str(columns{j}(2));
    
      % Build the remaing part of the directory/filename and the constraint
      directory = [ year(k,:) '/' ];
      img_name = [ 'atlas.ssmi.ver02.level3.5.' monstr(k,:) y2kyear(k,:) '.hdf' ];
      constraint = '';
      urlcon1 = [0 0 rows columns{j}];
      for i = 1:size(variable,1)
	if i > 1
	  constraint = [constraint ','];
	end
	constraint = [constraint deblank(variable(i,:)) ...
	      '[0:1:0]' ...
	      '[' StartRow ':' stride ':' EndRow ']' ...
	      '[' StartColumn ':' stride ':' EndColumn ']'];
      end
      iURL = [server directory img_name '?' constraint];
      if ~isempty(URL{k}), URL{k} = [URL{k} ' '];  end
      URL{k} = [URL{k} iURL];
      if k == 1
	urlcon = [urlcon; urlcon1];
      end
    end
  end

  urlcon = num2cell(urlcon,2);
  URL = char(URL);
  axindex{1} = [0 0];
  axindex{2} = [rows];
  axindex{3} = cat(1,columns{:});
  URLinfo.axindex = axindex(:);
  
  URLinfo.stride = [1; dset_stride; dset_stride];
  
  mapnames = cell(3,1);
  mapnames{1} = 'time';
  mapnames{2} = 'latitude';
  mapnames{3} = 'longitude';
  URLinfo.axnames = mapnames;
  
  URLinfo.urlconstraint = urlcon(:);
else
  URLinfo.axindex = cell(3,1);
  
  URLinfo.stride = [1; dset_stride; dset_stride];
  
  mapnames = cell(3,1);
  mapnames{1} = 'time';
  mapnames{2} = 'latitude';
  mapnames{3} = 'longitude';
  URLinfo.axnames = mapnames;
  
  URLinfo.urlconstraint = {};
end
return
