
// (c) COPYRIGHT URI/MIT 1994-2002
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      jhrg,jimg       James Gallagher <jgallagher@gso.uri.edu>
//      dan             Dan Holloway <dholloway@gso.uri.edu>
//      reza            Reza Nekovei <reza@intcomm.net>

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_dap.h"

static char rcsid[] not_used =
    { "$Id: Connect.cc,v 1.105.2.35 2002/08/22 21:23:23 jimg Exp $" };

#include <stdio.h>
#ifndef WIN32
#include <unistd.h>
#endif

#include <strstream>
#include <fstream>

#include "debug.h"
#include "DataDDS.h"
#include "Connect.h"
#include "escaping.h"
#include "RCReader.h"

#define SHOW_MSG (WWWTRACE || HTAlert_interactive())

using std::cerr;
using std::endl;
using std::ifstream;
using std::ofstream;

extern ObjectType get_type(const string &value);

/** This method should be able to stand on its own, along with similar
    methods for the DAS and DDS (maybe). That way Connect provides a default
    way to process a response plus an application can provide its own. */
void
Connect::process_data(DataDDS &data) throw(Error, InternalErr)
{
    switch (type()) {
      case dods_error:
	if (!_error.parse(output()))
	    throw InternalErr(__FILE__, __LINE__,
	      "Could not parse the Error object returned by the server!");
	throw _error;

      case web_error:
	// Web errors (those reported in the return document's MIME header)
	// are processed by the WWW library.
	return;

      case dods_data:
      default: {
	  // Parse the DDS; throw an exception on error.
	  data.parse(output());

	  // Load the DDS with data.
	  for (Pix q = data.first_var(); q; data.next_var(q)) {
	      data.var(q)->deserialize(source(), &data);
	  }
      }
    }
}

// This function returns the ObjectType value that matches the given string.
ObjectType 
get_type(const string &value)
{
    if (value == "dods_das")
	return dods_das;
    else if (value == "dods_dds")
	return dods_dds;
    else if (value == "dods_data")
	return dods_data;
    else if (value == "dods_error")
	return dods_error;
    else if (value == "web_error")
	return web_error;
    else
	return unknown_type;
}

// Barely a parser... This is used when reading from local sources of DODS
// Data objects. It simulates the important actions of the libwww MIME header
// parser. Those actions fill in certain fields in the Connect object. jhrg
// 5/20/97
//
// Make sure that this parser reads from data_source without disturbing the
// information in data_source that follows the MIME header. Since the DDS
// (which follows the MIME header) is parsed by a flex/bison scanner/parser,
// make sure to use I/O calls that will mesh with ANSI C I/O calls. In the
// old GNU libg++, the C++ calls were synchronized with the C calls, but that
// may no longer be the case. 5/31/99 jhrg

void 
Connect::parse_mime(FILE * data_source)
{
    char line[256];

    fgets(line, 255, data_source);
    line[strlen(line) - 1] = '\0';	// remove the newline

    while (line[0] != '\0') {
	char h[256], v[256];
	sscanf(line, "%s %s\n", h, v);
	string header = h;
	string value = v;
	downcase(header);
	downcase(value);

	if (header == "content-description:") {
	    DBG(cout << header << ": " << value << endl);
	    set_type(get_type(value));
	} else if (header == "xdods-server:") {
	    DBG(cout << header << ": " << value << endl);
	    set_server_version(value);
	} else if (server_version() == "dods/0.0" && header == "server:") {
	    DBG(cout << header << ": " << value << endl);
	    set_server_version(value);
	}

	fgets(line, 255, data_source);
	line[strlen(line) - 1] = '\0';
    }
}

// public mfuncs

Connect::Connect(string name, bool www_verbose_errors, bool accept_deflate, 
		 string uname, string password) throw (Error, InternalErr)
    : d_type(unknown_type), d_server(""), d_output(0)
{
    name = prune_spaces(name);
    
    // Figure out of the URL starts with 'http', if so, make sure that we
    // talk to an instance of HTTPConnect.
    if (name.find("http") == 0) {
	DBG(cerr << "Connect: The identifier is an http URL" << endl);
	d_http = new HTTPConnect(RCReader::instance());

	// Find and store any CE given with the URL.
	string::size_type dotpos = name.find('?');
	if (dotpos != name.npos) {
	    _URL = name.substr(0, dotpos);
	    string expr = name.substr(dotpos + 1);

	    dotpos = expr.find('&');
	    if (dotpos != expr.npos) {
		_proj = expr.substr(0, dotpos);
		_sel = expr.substr(dotpos);	// XXX includes '&'
	    } else {
		_proj = expr;
		_sel = "";
	    }
	} else {
	    _URL = name;
	    _proj = "";
	    _sel = "";
	}

	_local = false;
	_source = 0;
    } 
    else {
	DBG(cerr << "Connect: The identifier is a local data source." << endl);

	_URL = "";
	_local = true;		// local in this case means non-DODS
	_source = 0;
    }
}

Connect::~Connect()
{
    DBG2(cerr << "Entering the Connect dtor" << endl);

    if (!_local)
	delete d_http;

    close_output();

    DBG2(cerr << "Leaving the Connect dtor" << endl);
}

/** It should be an error to call this before a reponse has been generated. */
FILE *
Connect::output()
{
    if (_local)
	return d_output;
    else
	return d_http->output();
}

/** It should be an error to call this before a response has been generated. */
XDR *
Connect::source()
{
    if (!_source)
	_source = new_xdrstdio(output(), XDR_DECODE);

    return _source;
}

void 
Connect::close_output()
{
    if (d_output && _local) {
	fclose(d_output);
	d_output = 0;
    }

    if (_source) {
	delete_xdrstdio(_source);
	_source = 0;
    }
}

/** It should be an error... */
ObjectType 
Connect::type()
{
    if (d_type == unknown_type)
	d_type = d_http->type();
    return d_type;
}

/* Move to local access class when written. */
void
Connect::set_type(ObjectType ot)
{
    d_type = ot;
}

/** This method should only be called once a response has been requested and
    received. */
string 
Connect::server_version()
{
    if (d_server == "") {
	if (_local)
	    d_server = "dods/0.0";
	else 
	    d_server = d_http->server_version();
    }

    return d_server;
}

/** This method is here only because local response access is implemented ad
    a kludge. Once a class for local access is written, this methods should
    more there. */ 
void
Connect::set_server_version(const string &sv)
{
    d_server = sv;
}

void
Connect::request_das(DAS &das) throw(Error, InternalErr)
{
    string das_url = _URL + ".das";
    if (_proj.length() + _sel.length())
	das_url = das_url + "?" + id2www_ce(_proj + _sel);

    // We need to catch Error exceptions to ensure calling close_output.
    try {
	d_http->fetch_url(das_url);
    }
    catch (...) {
	close_output();
	throw;
    }

    switch (type()) {
      case dods_error: {
	  if (!_error.parse(output())) {
	      throw InternalErr(__FILE__, __LINE__, 
			"Could not parse error returned from server.");
	      break;
	  }
	  throw _error;
	  break;
      }

      case web_error:
	// We should never get here; a web error should be picked up read_url
	// (called by fetch_url) and result in a thrown Error object.
	break;

      case dods_das:
      default:
	// DAS::parse throws an exception on error.
	try {
	    das.parse(output());	// read and parse the das from a file 
	}
	catch (...) {
	    close_output();
	    throw;
	}
	    
	break;
    }

    close_output();
}

/** Read data and load it into a DAS object. This is how I'd like the
    interface for Connect to look; no more objects embedded in the class. */
void
Connect::request_dds(DDS &dds, string expr) throw(Error, InternalErr)
{
    string proj, sel;
    string::size_type dotpos = expr.find('&');
    if (dotpos != expr.npos) {
	proj = expr.substr(0, dotpos);
	sel = expr.substr(dotpos);
    } else {
	proj = expr;
	sel = "";
    }

    string dds_url = _URL + ".dds" + "?" 
	+ id2www_ce(_proj + proj + _sel + sel);

    // We need to catch Error exceptions to ensure calling close_output.
    try {
	d_http->fetch_url(dds_url);
    }
    catch (...) {
	close_output();
	throw;
    }

    switch (type()) {
      case dods_error: {
	  if (!_error.parse(output())) {
	      throw InternalErr(__FILE__, __LINE__, 
			"Could not parse error returned from server.");
	      break;
	  }
	  throw _error;
	  break;
      }

      case web_error:
	// We should never get here; a web error should be picked up read_url
	// (called by fetch_url) and result in a thrown Error object.
	break;

      case dods_dds:
      default:
	// DDS::prase throws an exception on error.
	try {
	    dds.parse(output());	// read and parse the dds from a file 
	}
	catch (...) {
	    close_output();
	    throw;
	}
	break;
    }

    close_output();
}

void
Connect::request_data(DataDDS &data, string expr) throw(Error, InternalErr)
{
    string proj, sel;
    string::size_type dotpos = expr.find('&');
    if (dotpos != expr.npos) {
	proj = expr.substr(0, dotpos);
	sel = expr.substr(dotpos);
    } else {
	proj = expr;
	sel = "";
    }

    string data_url = _URL + ".dods?" 
	+ id2www_ce(_proj + proj + _sel + sel);

    // We need to catch Error exceptions to ensure calling close_output.
    try {
	d_http->fetch_url(data_url);
	data.set_version(server_version());
	process_data(data);
    }
    catch (...) {
	close_output();
	throw;
    }
}

void
Connect::read_data(DataDDS &data, FILE *data_source) throw(Error, InternalErr)
{
    if (!data_source) 
	throw InternalErr(__FILE__, __LINE__, "data_source is null.");

    // Read from data_source and parse the MIME headers specific to DODS.
    parse_mime(data_source);

    data.set_version(server_version());

    d_output = data_source;

    try {
	process_data(data);
    }
    catch (...) {
	close_output();
	throw;
    }
}

bool 
Connect::is_local()
{
    return _local;
}

string 
Connect::URL(bool ce)
{
    if (_local)
	throw InternalErr(__FILE__, __LINE__, 
		  "URL(): This call is only valid for a DODS data source.");

    if (ce)
	return _URL + "?" + _proj + _sel;
    else
	return _URL;
}

string 
Connect::CE()
{
    if (_local)
	throw InternalErr(__FILE__, __LINE__, 
		  "CE(): This call is only valid for a DODS data source.");

    return _proj + _sel;
}

DAS & 
Connect::das()
{
    if (_local)
	throw InternalErr(__FILE__, __LINE__,
			  "Cannot access DAS for a non-DODS data source.");

    return _das;
}

DDS & 
Connect::dds()
{
    if (_local)
	throw InternalErr(__FILE__, __LINE__,
			  "Cannot access DDS for a non-DODS data source.");

    return _dds;
}

Error & 
Connect::error()
{
    return _error;
}

/** Set the credentials for responding to challenges while dereferencing
    URLs. 
    @param u The username.
    @param p The password. 
    @see extract_auth_info() */
void 
Connect::set_credentials(string u, string p)
{
    d_http->set_credentials(u, p);
}

/** Disable any further use of the client-side cache. In a future version
    of this software, this should be handled so that the www library is
    not initialized with the cache running by default. */
void
Connect::disable_cache()
{
}

/** Sets the #www_errors_To_stderr# property.

    @see is_www_errors_to_stderr
    @param state The state of the property. */
void 
Connect::set_www_errors_to_stderr(bool state)
{
}

/** Gets the state of the #www_errors_to_stderr# property. If TRUE this
    means that http errors will be printed to stderr in addition to being
    reported in the Error object. If FALSE only the Error object will be
    used. 

    @return TRUE if WWW errors should got to stderr, FALSE if only the
    Error object should be used. */
bool 
Connect::get_www_errors_to_stderr()
{
    return false;
}

/** Sets the list of accepted types. This string is meant to list all of
    the DODS datatypes that the client can grok and is sent to
    the server using the XDODS-Accept-Types MIME header. The server will
    try to only send back to the client datatypes that are listed. If the
    value of this header is `All', then the server assumes that the
    client can process all of the DODS datatypes. If only one or two
    types are \emph{not} understood, then they can be listed, each one
    prefixed by `!'. Thus, if a client does not understand `Sequences',
    it could set types to `!Sequences' as opposed to listing all of the
    DODS datatypes. Multiple values are separated by commas (,).

    Not all servers will honor this and some requests may not be possible
    to express with a very limited set of datatypes.

    NB: By default, the value `All' is used.

    @param types The string listing datatypes understood by this client. */ 
void 
Connect::set_accept_types(const string & types)
{
}

/** Gets the current string of `accepted types'. This string lists all of
    the DODS datatypes that the client can grok. 

    @see set_accepted_types
    @return A string listing the types this client declares to servers it
    can understand. */
string 
Connect::get_accept_types()
{
    return "All";
}

/** Set the cache control header value.
    @see get_cache_control
    @see HTTP/1.1 Specification, sec. 14.9.
    @param caching Should be no-cache to disable caching. */
void 
Connect::set_cache_control(const string & caching)
{
}

/** Get the string which describes the default cache control value. This
    is sent with all outgoing messages.<p>
    NB: The libwww 5.2.9 cache does not honor this.
    @return The cache control header value. */
string 
Connect::get_cache_control()
{
    return "";
}

// $Log: Connect.cc,v $
// Revision 1.105.2.35  2002/08/22 21:23:23  jimg
// Fixes for the Win32 Build made at ESRI by Vlad Plenchoy and myslef.
//
// Revision 1.105.2.34  2002/08/08 06:54:56  jimg
// Changes for thread-safety. In many cases I found ugly places at the
// tops of files while looking for globals, et c., and I fixed them up
// (hopefully making them easier to read, ...). Only the files RCReader.cc
// and usage.cc actually use pthreads synchronization functions. In other
// cases I removed static objects where they were used for supposed
// improvements in efficiency which had never actually been verifiied (and
// which looked dubious).
//
// Revision 1.105.2.33  2002/07/06 19:33:34  jimg
// I fixed some of the access functions that provide information that servers
// send in the headers of a response. If we're accessing a local response (a
// DataDDS dumped to a file or read from stdin) the behavior of HTTP is
// simulated (something that we should change but can stay for now). In some
// cases it was possible for a client of Connect to ask the HTTPConnect object
// for information when that object had not been initialized because the data
// source was a local file. The implementation of Connect still needs work...
//
// Revision 1.105.2.32  2002/06/21 22:23:29  jimg
// I revised the basic interface to this class. The old interface is implemented
// using the new one. Methods to get objects from DODS objects now take
// references to the containers for those objects as formal parameters. This
// enables a user of Connect to control how/when/where those containers are
// created and managed.
//
// Revision 1.105.2.31  2002/06/21 00:12:51  jimg
// Added a new request_dds() method that takes a reference to a DDS and dumps
// the retrieved info into it instead of the DDS that's held locally by this
// class. This is a much more flexible approach to reading responses. I also
// reimplemented the older version of request_dds using the new method (easy
// since the GUI stuff is disabled).
//
// Revision 1.105.2.30  2002/06/20 06:21:27  jimg
// Fixed local access. For now access to OPENDAP objects stored in local files
// uses a local FILE *. Methods test _local to determine whether to use the
// HTTPConnect object or the local FILE *.
//
// Revision 1.105.2.29  2002/06/20 03:18:48  jimg
// Fixes and modifications to the Connect and HTTPConnect classes. Neither
// of these two classes is complete, but they should compile and their
// basic functions should work.
//
// Revision 1.105.2.28  2002/06/18 23:03:43  jimg
// Added extern declaration of ObjectType get_type(...).
//
// Revision 1.105.2.27  2002/06/18 21:56:16  jimg
// Implementation of the new Connect class. HTTP is no longer swimming around
// inside this code. However, a lot of the methods need tweaing and the
// interface needs to be reworked (again).
//
