
// -*- C++ -*-

// (c) COPYRIGHT URI/MIT 1996,1998,1999
// Please first read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//	jhrg,jimg	James Gallagher (jgallagher@gso.uri.edu)

// Graphical User Interface object. Starts the GUI in a separate process. The
// GUI must read commands from stdin and write responses to stdout. Users can
// signal that they do not want the GUI using an environment variable
// (DODS_USE_GUI) and can use a different GUI (specified with the DODS_GUI
// environment variable) so long as it conforms to the simple protocol of the
// default GUI. The GUI is sent the command in the DODS_GUI_INIT environment
// variable once it is started (but before any other commands are issued). If
// no such command is required, define this to be an empty string.
//
// jhrg 6/21/96

#ifndef _gui_h
#define _gui_h

#ifdef __GNUG__
#pragma interface
#endif

#include <string>

/** This class manages a simple graphical user interface (GUI) for a
    DODS client. The GUI is implemented by an embedded Tcl/Tk interpreter and
    a short tk script (see dods_gui.tcl). The script contains a number of
    functions which are loaded into the embedded interpreter. The functions
    defined by the script can be run using the methods of Gui. Some of the
    functions can be accessed only through certain special methods (e.g.,
    simple_error) while others can be accessed using their name via the
    command(...) method.

    *** Reimplement as a singleton ***

    The Gui class was/is intended to support the transfer progress indicator.
    It also supports display of various error messages and provides a way for
    users to supply username and password pairs for secure sites.

    Do not confuse the DODS Client GUI, managed by this class, with a
    DODS GUI Client, such as the DODS Matlab GUI, used for the
    graphical display of DODS datasets.  The two can work together,
    but otherwise have nothing to do with each other.

    The GUI process uses the following environment variables:
    
    \begin{description}

    \item{#DODS_USE_GUI#} If this variable exists and equals ``no'',
    the progress indicator will not appear. If it does not exist, or
    if it has any other value, the progress indicator will be
    activated. 

    \end{description}

    Note that for historical reasons there is no copy constructor or compare
    operators. 

    @memo Implements a graphic user interface for a DODS client. */

class Gui {
private:
    bool _show_gui;		// True if we should use the GUI.
    bool _user_wants_gui;	// True if the user wants the GUI.
    bool _progress_visible;	// True when the PI is mapped.

    string _cmd;		// Command used to create object.

    void clone(const Gui &) {}
    Gui(const Gui &) {}
    Gui &operator=(const Gui &) {return *this;}

public:
    /** Create the instance of Gui and initialize the embedded interpreter. In
	previous versions of this class it was possible to change the way the
	interpreter was initialized. However, that is no longer an option.

	If the constructor is called with a string argument, send that string
	to the interpreter as an initial command.

	@name Gui();
	@param cmd Send the Tcl/Tk interpreter this command.
	@memo The Gui class constructor. */
    //@{
    ///
    Gui(const string cmd);
    ///
    Gui();
    //@}
    virtual ~Gui();

    /** This function can be used to query whether the progress
	indicator is visible or not.  It may also be used to set whether
	it is to be seen or not.

	@memo Set or test progress indicator visibility.
	@param state If {\it state} is omitted (or is -1, the default), 
	the function returns the visibility state.  If {\it state} is 
	included, the function sets the visibility state to the given 
	value.
	@return Returns TRUE if the GUI is visible, false otherwise. */
    bool progress_visible(int state = -1);

    /** This function returns the state of two internal flags.  The
	first, under program control, indicates whether or not to use
	the GUI at all.  This is a private value called #_show_gui#.
	The second flag is derived from the value of the environment
	variable #DODS_USE_GUI#, and is set when the class instance is
	created.  It is also a private value (#_user_wants_gui#), and
	cannot be modified.  The GUI is displayed only if both these
	flags are TRUE.

	This function can be used to modify the value of the #_show_gui#
	flag.

	@memo Should the GUI be used? 
	@param state If {\it state} is omitted (or is -1, the default),
	the function returns the result of the operation #_show_gui# AND
	#_user_wants_gui#.  If {\it state} is included, the function
	sets the value of the #_show_gui# flag.
	@return The value of #_show_gui# AND #_user_wants_gui#. */
    bool show_gui(int state = -1);

    /** Submits a command to the GUI interpreter for execution.
 
	It may be that users turn off the GUI (because they are sick of
	it).  In that case, this command will simply return false. It
	does no harm to call the command when no GUI is running. Care
	should be taken to send only valid commands---invalid commands
	may put the GUI into an inoperable state.
    
	If the GUI is not running, this function starts it.

	@memo Send the GUI process a command.
	@return TRUE if the command succeeded, FALSE otherwise.
	@param command A command to submit to the GUI process for
	execution. */
    bool command(string command);

    /** Pops up a username/password dialog box. The values entered by the
	user are returned in the parameters #user# and #pass#.

	@memo Get username and password information.
	@param user The user's name.
	@param pass The user's password.
	@return true if the command succeeded, false otherwise. */
    bool password(string &user, string &pass);

    /** Print an error message and wait for the user to acknowledge it.

	@param msg  message to be printed. */
    bool simple_error(string msg);

    /** moves the progess bar back and forth in its slot. If usr presses
        "Cancel" button parameter #usr_cancel# is true

        @ assign value to progess bar, get Cancel button value.
        @ param pro Percent of progress.
        @ param usr_cancel The cancel button.        */
    bool percent_bar(double pro, int *usr_cancel);

};

// $Log: Gui.h,v $
// Revision 1.14.4.1  2001/10/08 17:12:13  jimg
// Fixed up the comments.
//
// Revision 1.14  2000/09/22 02:17:20  jimg
// Rearranged source files so that the CVS logs appear at the end rather than
// the start. Also made the ifdef guard symbols use the same naming scheme and
// wrapped headers included in other headers in those guard symbols (to cut
// down on extraneous file processing - See Lakos).
//
// Revision 1.13  2000/08/02 22:46:49  jimg
// Merged 3.1.8
//
// Revision 1.9.2.1  2000/08/01 21:09:35  jimg
// Destructor is now virtual
//
// Revision 1.12  2000/04/07 00:19:05  jimg
// Merged Brent's changes for the progress gui - he added a cancel button.
// Also repaired the last of the #ifdef Gui bugs so that we can build Gui
// and non-gui versions of the library that use one set of header files.
//
// Revision 1.11.4.1  2000/04/04 05:00:24  brent
// put a Cancel button the Tcl/Tk GUI
//
// Revision 1.11  1999/12/31 01:00:23  jimg
// Spelling fixes...
//
// Revision 1.10  1999/12/31 00:55:10  jimg
// Fixed up the progress indicator
//
// Revision 1.9  1999/08/09 18:27:34  jimg
// Merged changes from Brent for the Gui code (progress indicator)
//
// Revision 1.8.8.1  1999/07/29 05:46:17  brent
// call Tcl / GUI directly from Gui.cc, abandon expect, and consolidate Tcl files
//
// Revision 1.8  1999/04/29 02:29:30  jimg
// Merge of no-gnu branch
//
// Revision 1.7  1998/11/23 15:05:55  tom
// late modifications to documentation
//
// Revision 1.6.6.1  1999/02/02 21:56:59  jimg
// String to string version
//
// Revision 1.6  1998/01/14 22:55:04  tom
// First draft of doc++ class documentation.
//
// Revision 1.5  1996/12/18 19:16:12  jimg
// Include assert.h. Arrgh!
//
// Revision 1.4  1996/08/13 18:29:21  jimg
// Added String.h - a bit of a retrenchment from my earlier `No nested
// includes' policy...
//
// Revision 1.3  1996/07/09 23:24:08  jimg
// Modified operator=() so that it passes -Wall (removed unused formal
// parameter RHS and provided a return value (*THIS).
//
// Revision 1.2  1996/07/08 23:18:36  jimg
// Added support for the HAVE_EXPECT compile-time switch. When defined as 1
// support for expect is built, when 0 it is not. Thus the dap++ library may be
// built so that expect is not required to link programs. However, the GUI will
// work only when expect/tcl are present and when client are linked with those
// libraries in addition to libdap++.a
// Fixed comments.
// Fixed some warnings (but left some...).
//
// Revision 1.1  1996/06/21 23:13:09  jimg
// Created based on code from Connect.h.
//

#endif // _gui_h
